PROVINCES = 'provincias'
DEPARTMENTS = 'departamentos'
MUNICIPALITIES = 'municipios'
SETTLEMENTS = 'asentamientos'
LOCALITIES = 'localidades'
CENSUS_LOCALITIES = 'localidades_censales'
STREETS = 'calles'
INTERSECTIONS = 'intersecciones'
STREET_BLOCKS = 'cuadras'
SYNONYMS = 'sinonimos'
EXCLUDING_TERMS = 'terminos_excluyentes'

ENTITIES = [
    'provincias',
    'departamentos',
    'municipios',
    'localidades-censales',
    'asentamientos',
    'localidades',
    'calles'
]

PROVINCES_DICT = {
    '02': 'Ciudad Autónoma de Buenos Aires',
    '06': 'Buenos Aires',
    '10': 'Catamarca',
    '14': 'Córdoba',
    '18': 'Corrientes',
    '22': 'Chaco',
    '26': 'Chubut',
    '30': 'Entre Ríos',
    '34': 'Formosa',
    '38': 'Jujuy',
    '42': 'La Pampa',
    '46': 'La Rioja',
    '50': 'Mendoza',
    '54': 'Misiones',
    '58': 'Neuquén',
    '62': 'Río Negro',
    '66': 'Salta',
    '70': 'San Juan',
    '74': 'San Luis',
    '78': 'Santa Cruz',
    '82': 'Santa Fe',
    '86': 'Santiago del Estero',
    '90': 'Tucumán',
    '94': 'Tierra del Fuego, Antártida e Islas del Atlántico Sur'
}
